"""
Module contains functions related to SSH.

Most of this script is adapted from:
https://github.com/paramiko/paramiko/blob/master/demos/forward.py
"""
import webbrowser
import select
import os
import socketserver

from paramiko import RSAKey, SSHClient, AutoAddPolicy
from paramiko.ssh_exception import SSHException

from bokchoi import common


class ForwardServer(socketserver.ThreadingTCPServer):
    daemon_threads = True
    allow_reuse_address = True


class Handler(socketserver.BaseRequestHandler):

    ssh_transport = None
    host_port = None

    def __init__(self, request, client_address, server):
        super(Handler, self).__init__(request, client_address, server)

    def handle(self):

        channel = self.ssh_transport.open_channel('direct-tcpip',
                                                 ('localhost', 8888),
                                                 ('localhost', self.host_port))
        if not channel:
            print('Incoming request was rejected')
            return

        while True:
            r, _, _ = select.select([self.request, channel], [], [])
            if self.request in r:
                data = self.request.recv(1024)
                if not data:
                    break
                channel.send(data)
            if channel in r:
                data = channel.recv(1024)
                if not data:
                    break
                self.request.send(data)

        channel.close()
        self.request.close()


def forward(local_port, remote_host, user_name, key_filename):
    """ Sets up port forwarding to remote host
    :param local_port:              Local port to forward to
    :param remote_host:             Remote host
    :param user_name:               User to use in ssh connection
    :param key_filename:            Private key for ssh connection
    :return:                        -
    """
    client = SSHClient()
    client.load_system_host_keys()
    client.set_missing_host_key_policy(AutoAddPolicy())

    print('Connecting to ssh host {}:{} ...'.format(remote_host, 8888))

    common.retry(client.connect
                 , SSHException
                 , hostname=remote_host
                 , port=22
                 , username=user_name
                 , key_filename=key_filename)

    class SubHandler(Handler):
        ssh_transport = client.get_transport()
        host_port = local_port

    server = ForwardServer(('localhost', local_port), SubHandler)

    try:
        webbrowser.open('http://localhost:{}'.format(local_port))
        server.serve_forever()
    except KeyboardInterrupt:
        server.shutdown()
        server.server_close()
        print('Connection closed')


def get_ssh_keys(project_id):
    """Get private and public keys. Create if not exists.
    :param project_id:          Project id used for private key name
    :return:                    -
    """
    ssh_dir = os.path.join(os.path.expanduser('~'), '.ssh')
    if not os.path.exists(ssh_dir):
        os.makedirs(ssh_dir)

    key_file = os.path.join(ssh_dir, project_id)
    try:
        priv = RSAKey.from_private_key_file(key_file)
    except FileNotFoundError:
        print('Could not find private key. Creating one.')
        priv = RSAKey.generate(2048)
        priv.write_private_key_file(key_file)
    pub = priv.get_base64()
    return pub




import os
import zipfile
from io import BytesIO

import boto3
from botocore.exceptions import ClientError

from bokchoi import common

session = boto3.Session()

s3_client = session.client('s3')
s3_resource = session.resource('s3')

lambda_client = session.client('lambda')

events_client = session.client('events')

SCHEDULER_TRUST_POLICY = """{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "",
      "Effect": "Allow",
      "Principal": {
        "Service": "lambda.amazonaws.com"
      },
      "Action": "sts:AssumeRole"
    }
  ]
}"""


SCHEDULER_POLICY = """{
  "Version": "2012-10-17",
  "Statement": [{
    "Effect": "Allow",
    "Action": [
       "ec2:DescribeImages",
       "ec2:DescribeSubnets",
       "ec2:RequestSpotInstances",
       "ec2:TerminateInstances",
       "ec2:DescribeInstanceStatus",
       "ec2:DescribeSecurityGroups",
       "ec2:DescribeSpotInstanceRequests",
       "ec2:CreateTags",
       "iam:PassRole",
       "elasticmapreduce:RunJobFlow",
       "elasticmapreduce:AddJobFlowSteps"
        ],
    "Resource": ["*"]
  }]
}"""

EVENT_POLICY = """{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Effect": "Allow",
      "Action": [
        "lambda:InvokeFunction"
      ],
      "Resource": "*"
    }
  ]
}"""

EVENT_TRUST_POLICY = """{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "",
      "Effect": "Allow",
      "Principal": {
        "Service": "events.amazonaws.com"
      },
      "Action": "sts:AssumeRole"
    }
  ]
}"""


class Scheduler:

    def __init__(self, project_id, project_name, schedule, requirements=None):

        self.project_id = project_id
        self.project_name = project_name

        self.schedule = schedule

        self.bucket_name = project_id

        self.requirements = requirements

        self.function_name = project_id + '-scheduler'
        self.rule_name = project_id + '-rule'

    def deploy(self):

        file_object = self.zip_package(self.requirements)

        zip_file_name = 'bokchoi-scheduler.zip'
        common.upload_to_s3(self.bucket_name, file_object, zip_file_name, fingerprint='n/a')

        scheduler_role = self.create_role(self.project_id
                                          , self.project_id + '-scheduler-role', SCHEDULER_TRUST_POLICY
                                          , self.project_id + '-scheduler-policy', SCHEDULER_POLICY)

        lambda_function = common.retry(self.create_lambda_function
                                       , ClientError
                                       , project_id=self.project_id
                                       , project_name=self.project_name
                                       , role=scheduler_role
                                       , bucket_name=self.bucket_name
                                       , zip_file_name=zip_file_name)

        event_role = self.create_role(self.project_id
                                      , self.project_id + '-event-role', EVENT_TRUST_POLICY
                                      , self.project_id + '-event-policy', EVENT_POLICY)

        self.create_cloudwatch_rule(self.schedule, event_role.arn, lambda_function['FunctionArn'])

    def undeploy(self, dryrun):

        self.delete_lambda_function(dryrun)
        self.delete_cloudwatch_rule(dryrun)

    def create_role(self, project_id, role_name, trust_policy, policy_name, policy):
        common.create_policy(policy_name, policy)
        scheduler_policy = next(common.get_policies(project_id, pattern=policy_name))

        return common.create_role(role_name, trust_policy, scheduler_policy)

    def zip_package(self, requirements=None):

        from bokchoi import ec2, emr, common, scheduler_lambda

        file_object = BytesIO()

        with zipfile.ZipFile(file_object, 'w', zipfile.ZIP_DEFLATED) as zip_file:
            zip_file.write(scheduler_lambda.__file__, 'scheduler_lambda.py')
            zip_file.write(common.__file__, 'bokchoi/common.py')
            zip_file.write(ec2.__file__, 'bokchoi/ec2.py')
            zip_file.write(emr.__file__, 'bokchoi/emr.py')
            zip_file.write(emr.__file__, 'bokchoi/scheduler.py')
            zip_file.write('/'.join((os.getcwd(), 'bokchoi_settings.json')), 'bokchoi_settings.json')

            zip_file.writestr('__init__.py', '')
            info = zipfile.ZipInfo('bokchoi/__init__.py')
            info.external_attr = 0o777 << 16  # give full access to included file
            zip_file.writestr(info, '')

            requirements = requirements or ''
            zip_file.writestr('requirements.txt', '\n'.join(requirements))

        file_object.seek(0)

        return file_object

    def create_lambda_function(self, project_id, project_name, role, bucket_name, zip_file_name):

        try:
            lambda_client.create_function(FunctionName=self.function_name
                                          , Runtime='python3.6'
                                          , Handler='scheduler_lambda.run'
                                          , Role=role.arn
                                          , Code={'S3Bucket': bucket_name
                                                  , 'S3Key': zip_file_name}
                                          , Timeout=30
                                          , Environment={'Variables': {'project_name': project_name}}
                                          , Tags={'bokchoi-id': project_id})
        except ClientError as e:
            if e.response['Error']['Code'] == 'ResourceConflictException':
                print('Scheduler already exists')
            else:
                raise e

        return lambda_client.get_function_configuration(FunctionName=self.function_name)

    def delete_lambda_function(self, dryrun):
        """ Deletes scheduler lambda function"""

        if dryrun:
            print('Dryrun flag set. Would have deleted scheduler')
            return

        try:
            lambda_client.delete_function(FunctionName=self.function_name)
        except ClientError as e:
            if e.response['Error']['Code'] == 'ResourceNotFoundException':
                print('Function does not exist')
            else:
                raise e

    def create_cloudwatch_rule(self, schedule, role_arn, function_arn):
        """ Creates Cloudwatch rule (event) that will trigger lambda function
        :param schedule:                Event schedule
        :param role_arn:                ARN of role to assign to rule
        :param function_arn:            ARN of Lambda function to trigger
        """
        print('Scheduling job using ' + schedule)

        common.retry(events_client.put_rule
                     , ClientError
                     , Name=self.rule_name
                     , ScheduleExpression=schedule
                     , State='ENABLED'
                     , RoleArn=role_arn)

        events_client.put_targets(Rule=self.rule_name
                                  , Targets=[{'Arn': function_arn, 'Id': '0'}])

        try:
            lambda_client.add_permission(
                FunctionName=self.function_name,
                StatementId='0',
                Action='lambda:InvokeFunction',
                Principal='events.amazonaws.com'
            )
        except ClientError as e:
            if e.response['Error']['Code'] == 'ResourceConflictException':
                print('Scheduler rule already exists')
            else:
                raise e

    def delete_cloudwatch_rule(self, dryrun):
        """ Delete Cloudwatch rule (event). First removes all targets"""

        if dryrun:
            print('Dryrun flag set. Would have deleted cloudwatch rule')
            return

        try:
            events_client.remove_targets(Rule=self.rule_name
                                         , Ids=['0'])
        except ClientError as e:
            if e.response['Error']['Code'] == 'ResourceNotFoundException':
                print('No targets to remove from rule')
            else:
                raise e

        try:
            events_client.delete_rule(Name=self.rule_name)
        except ClientError as e:
            if e.response['Error']['Code'] == 'ResourceNotFoundException':
                print('Cloudwatch rule does not exist')
            else:
                raise e



#!/usr/bin/env python3.6
"""
Class which can be used to deploy and run EMR jobs
"""

import os
import sys
import time
import boto3

from bokchoi import common


class EMR(object):
    """Create EMR object which can be used to schedule jobs"""
    def __init__(self, project, settings):
        self.settings = settings
        self.project_name = project

        aws_account_id = common.get_aws_account_id()
        self.project_id = common.create_project_id(project, aws_account_id)
        self.job_flow_id = None

    def schedule(self):
        """Schedule task"""
        if self.settings.get('Schedule'):

            from bokchoi.scheduler import Scheduler

            scheduler = Scheduler(self.project_id
                                  , self.project_name
                                  , self.settings.get('Schedule')
                                  , self.settings.get('Requirements'))
            scheduler.deploy()

    def deploy(self):
        """Zip package and deploy to S3 so it can be used by EMR"""
        bucket_name = common.create_bucket(self.settings['Region'], self.project_id)

        cwd = os.getcwd()
        package, fingerprint = common.zip_package(cwd, self.settings.get('Requirements'))

        package_name = 'bokchoi-' + self.project_name + '.zip'
        common.upload_to_s3(bucket_name, package, package_name, fingerprint)
        self.schedule()

    def run(self):
        """Create Spark cluster and run specified job"""
        emr_client = boto3.client('emr')
        self.start_spark_cluster(emr_client)
        self.step_prepare_env(emr_client)
        self.step_spark_submit(emr_client)

    def undeploy(self, dryrun):
        """Deletes all policies, users, and instances permanently"""

        common.cancel_spot_request(self.project_id, dryrun)
        common.terminate_instances(self.project_id, dryrun)

        for pol in common.get_policies(self.project_id):
            common.delete_policy(pol, dryrun)

        for prof in common.get_instance_profiles(self.project_id):
            common.delete_instance_profile(prof, dryrun)

        for role in common.get_roles(self.project_id):
            common.delete_role(role, dryrun)

        # remove s3 bucket
        common.delete_bucket(self.project_id, dryrun)

        from bokchoi.scheduler import Scheduler

        scheduler = Scheduler(self.project_id
                              , self.project_name
                              , self.settings.get('Schedule')
                              , self.settings.get('Requirements'))
        scheduler.undeploy(dryrun)

    def start_spark_cluster(self, emr_client):
        """
        Start Spark cluster based on configuration given in settings
        """

        launch_spec = self.settings['EMR']['LaunchSpecification']

        instance_type = launch_spec['InstanceType']
        instance_count = self.settings['EMR']['InstanceCount']

        instances = {'KeepJobFlowAliveWhenNoSteps': False
                     , 'TerminationProtected': False
                     , 'Ec2SubnetId': self.settings['EMR']['LaunchSpecification']['SubnetId']
                     , 'InstanceGroups': [
                            {
                                'Name': 'EmrMaster',
                                'Market': 'SPOT',
                                'InstanceRole': 'MASTER',
                                'BidPrice': self.settings['EMR']['SpotPrice'],
                                'InstanceType': instance_type,
                                'InstanceCount': 1
                            },
                            {
                                'Name': 'EmrCore',
                                'Market': 'SPOT',
                                'InstanceRole': 'CORE',
                                'BidPrice': self.settings['EMR']['SpotPrice'],
                                'InstanceType': instance_type,
                                'InstanceCount': instance_count - 1
                            }
                        ]
                     }

        additional_sgs = launch_spec.get('AdditionalSecurityGroups')
        if additional_sgs:
            instances['AdditionalMasterSecurityGroups'] = additional_sgs
            instances['AdditionalSlaveSecurityGroups'] = additional_sgs

        response = emr_client.run_job_flow(
            Name=self.project_id,
            LogUri="s3://{}/spark/".format(self.project_id),
            ReleaseLabel=self.settings['EMR']['Version'],
            Instances=instances,
            Configurations=[
                {"Classification": "spark-env"
                 , "Properties": {}
                 , "Configurations": [
                        {
                            "Classification": "export",
                            "Properties": {
                                "PYSPARK_PYTHON": "python34"
                            },
                            "Configurations": []
                        }
                    ]
                }
            ],
            Tags=[{'Key': 'bokchoi-id', 'Value': self.project_id}],
            Applications=[{'Name': 'Hadoop'}, {'Name': 'Spark'}],
            JobFlowRole='EMR_EC2_DefaultRole',
            ServiceRole='EMR_DefaultRole',
            VisibleToAllUsers=True,
        )
        # parse EMR response to check if successful
        response_code = response['ResponseMetadata']['HTTPStatusCode']
        if response['ResponseMetadata']['HTTPStatusCode'] == 200:
            self.job_flow_id = response['JobFlowId']
        else:
            print("Error creating: (status code {})".format(response_code))
            sys.exit(1)

        print("Created Spark cluster with job {}".format(self.job_flow_id))

    def step_prepare_env(self, emr_client):
        """Copies files from S3 and unzips them"""
        package_name = 'bokchoi-' + self.project_name + '.zip'
        s3_package_uri = 's3://{bucket}/{key}'.format(bucket=self.project_id, key=package_name)
        root_dir = '/home/hadoop/'

        emr_client.add_job_flow_steps(
            JobFlowId=self.job_flow_id,
            Steps=[
                {
                    'Name': 'setup - copy files',
                    'ActionOnFailure': 'CANCEL_AND_WAIT',
                    'HadoopJarStep': {
                        'Jar': 'command-runner.jar',
                        'Args': ['aws', 's3', 'cp', s3_package_uri, root_dir]
                    }
                },
                {
                    'Name': 'setup - unzip files',
                    'ActionOnFailure': 'CANCEL_AND_WAIT',
                    'HadoopJarStep': {
                        'Jar': 'command-runner.jar',
                        'Args': ['unzip', root_dir + package_name, '-d', root_dir]
                    }
                },
                {
                    'Name': 'setup - install python dependencies',
                    'ActionOnFailure': 'CANCEL_AND_WAIT',
                    'HadoopJarStep': {
                        'Jar': 'command-runner.jar',
                        'Args': ['sudo', 'pip-3.4', 'install', '-r', root_dir + 'requirements.txt']
                    }
                }
            ]
        )

    def step_spark_submit(self, emr_client):
        """Submit spark job given by user"""
        py3_env = 'PYSPARK_PYTHON=/usr/bin/python3'
        script = self.settings['EntryPoint']

        emr_client.add_job_flow_steps(
            JobFlowId=self.job_flow_id,
            Steps=[
                {
                    'Name': 'Run Spark',
                    'ActionOnFailure': 'CANCEL_AND_WAIT',
                    'HadoopJarStep': {
                        'Jar': 'command-runner.jar',
                        'Args': ['spark-submit', '--conf', py3_env, '/home/hadoop/' + script]
                    }
                }
            ]
        )
        print("Added step 'spark-submit'")
        time.sleep(1)  # Prevent ThrottlingException



#!/usr/bin/env python3.6
"""
Main cli program which allows execution of commands
"""
import sys
import click

from bokchoi import common
from bokchoi.ec2 import EC2
from bokchoi.emr import EMR


@click.group()
@click.argument('project')
@click.pass_context
def cli(ctx, project):
    """Starting point for cli commands"""
    settings = common.load_settings(project)

    # instantiate object based on given platform parameter in settings file
    platforms = {'EC2': EC2, 'EMR': EMR}
    ctx.obj = platforms.get(settings['Platform'])(project, settings)

    if not ctx.obj:
        click.echo("Choose a supported instance type option..")
        sys.exit(1)


@cli.command('run')
@click.pass_obj
def run(ctx):
    """Default run command"""
    ctx.run()
    click.secho('Running application', fg='green')


@cli.command('deploy')
@click.pass_obj
def deploy(ctx):
    """Default deploy command"""
    ctx.deploy()
    click.secho('Deployed', fg='green')


@cli.command('undeploy')
@click.option('--dryrun', is_flag=True, default=False, help="Print in stead of terminate")
@click.pass_obj
def undeploy(ctx, dryrun):
    """Default undeploy command"""
    ctx.undeploy(dryrun)
    click.secho('Undeployed', fg='green')


@cli.command('connect')
@click.pass_obj
def connect(ctx):
    """Default connect command"""
    ctx.connect()


@cli.command('stop')
@click.pass_obj
def stop(ctx):
    """Default stop command"""
    ctx.stop()



#!/usr/bin/env python3.6
"""
Class which can be used to deploy and run EC2 spot instances
"""
from base64 import b64encode
import os

from bokchoi import common, ssh

USER_DATA = """#!/bin/bash

# Install aws-cli
sudo curl "https://s3.amazonaws.com/aws-cli/awscli-bundle.zip" -o "awscli-bundle.zip"
python3 -c "import zipfile; zf = zipfile.ZipFile('/awscli-bundle.zip'); zf.extractall('/');"
sudo chmod u+x /awscli-bundle/install
python3 /awscli-bundle/install -i /usr/local/aws -b /usr/local/bin/aws

# Download project zip
aws s3 cp s3://{bucket}/{package} /tmp/
python3 -c "import zipfile; zf = zipfile.ZipFile('/tmp/{package}'); zf.extractall('/tmp/');"

# Install pip3 and install requirements.txt from project zip if included
curl -sS https://bootstrap.pypa.io/get-pip.py | sudo python3
[ -f /tmp/requirements.txt ] && pip3 install -r /tmp/requirements.txt

if [ "{notebook}" = "True" ]
then
    #Add public key
    echo "ssh-rsa {public_key}" >> /home/ubuntu/.ssh/authorized_keys
    #Install Jupyter
    pip3 install jupyter
    pip3 install jupyterlab
    jupyter serverextension enable --py jupyterlab --sys-prefix
    pip3 install tornado==4.5.2
    echo "c.NotebookApp.token = u''" >> ~/.jupyter/jupyter_notebook_config.py
    jupyter lab --no-browser --allow-root --ip=0.0.0.0 --port=8888 --NotebookApp.token=
else
    # Run app
    cd /tmp
    python3 -c "import {app}; {app}.{entry}();"
    aws s3 cp /var/log/cloud-init-output.log s3://{bucket}/cloud-init-output.log
    if [ "{shutdown}" = "True" ]
    then
        shutdown -h now
    fi
fi
"""

DEFAULT_TRUST_POLICY = """{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "",
      "Effect": "Allow",
      "Principal": {
        "Service": "ec2.amazonaws.com"
      },
      "Action": "sts:AssumeRole"
    }
  ]
}"""

DEFAULT_POLICY = """{{
  "Version": "2012-10-17",
  "Statement": [
    {{
      "Action": [
        "s3:Get*",
        "s3:List*",
        "s3:Put*"
      ],
      "Effect": "Allow",
      "Resource": "arn:aws:s3:::{bucket}/*"
    }}
  ]
}}"""


class EC2(object):
    """Create EC2 object which can be used to schedule jobs"""
    def __init__(self, project, settings):
        self.settings = settings
        self.project_name = project
        self.requirements = settings.get('Requirements')
        self.region = settings.get('Region')
        self.entry_point = settings.get('EntryPoint')
        self.launch_config = settings.get('EC2')
        self.custom_policy = settings.get('CustomPolicy')

        self.shutdown = settings.get('Shutdown', True)

        aws_account_id = common.get_aws_account_id()
        self.project_id = common.create_project_id(project, aws_account_id)
        self.package_name = 'bokchoi-' + self.project_name + '.zip'

        self.schedule = settings.get('Schedule')

    def deploy(self):
        """Zip package and deploy to S3"""

        bucket_name = common.create_bucket(self.region, self.project_id)

        cwd = os.getcwd()
        package, fingerprint = common.zip_package(cwd, self.requirements)
        common.upload_to_s3(bucket_name, package, self.package_name, fingerprint)

        policies = self.create_policies(self.custom_policy)

        self.create_default_role_and_profile(policies)

        if self.settings.get('Notebook'):
            subnet = common.get_subnet(self.launch_config['LaunchSpecification']['SubnetId'])
            common.create_security_group(self.project_id + '-default'
                                         , self.project_id
                                         , subnet.vpc_id
                                         , {'CidrIp': common.get_my_ip() + '/32'
                                            , 'FromPort': 22
                                            , 'ToPort': 22
                                            , 'IpProtocol': 'tcp'}
                                         )

        if self.settings.get('Schedule'):

            from bokchoi.scheduler import Scheduler

            scheduler = Scheduler(self.project_id
                                  , self.project_name
                                  , self.settings.get('Schedule')
                                  , self.settings.get('Requirements'))
            scheduler.deploy()

    def undeploy(self, dryrun):
        """Deletes all policies, users, and instances permanently"""

        common.cancel_spot_request(self.project_id, dryrun)

        for instance in common.get_instances(self.project_id):
            common.terminate_instance(instance, dryrun)

        common.delete_bucket(self.project_id, dryrun)

        for policy in common.get_policies(self.project_id):
            common.delete_policy(policy, dryrun)

        for instance_profile in common.get_instance_profiles(self.project_id):
            common.delete_instance_profile(instance_profile, dryrun)

        for role in common.get_roles(self.project_id):
            common.delete_role(role, dryrun)

        for group in common.get_security_groups(self.project_id):
            common.delete_security_group(group, dryrun)

        from bokchoi.scheduler import Scheduler

        scheduler = Scheduler(self.project_id
                              , self.project_name
                              , self.settings.get('Schedule')
                              , self.settings.get('Requirements'))
        scheduler.undeploy(dryrun)

    def run(self):
        """Create EC2 machine with given AMI and instance settings"""
        print("Starting EC2 instance")

        bucket_name = self.project_id

        if self.settings.get('Notebook'):
            public_key = ssh.get_ssh_keys(self.project_id)
        else:
            public_key = ''

        if self.entry_point:
            app, entry = self.entry_point.split('.')
        else:
            app, entry = '', ''

        user_data = USER_DATA.format(bucket=bucket_name
                                     , package=self.package_name
                                     , app=app
                                     , entry=entry
                                     , public=public_key
                                     , shutdown=self.shutdown
                                     , notebook=self.settings.get('Notebook')
                                     , public_key=public_key)

        self.launch_config['LaunchSpecification']['UserData'] = b64encode(user_data.encode('ascii')).decode('ascii')

        self.launch_config['LaunchSpecification']['IamInstanceProfile'] = {'Name': self.project_id + '-default-role'}

        if self.settings.get('Notebook'):
            security_group = next(common.get_security_groups(self.project_id, self.project_id + '-default'))

            if self.launch_config['LaunchSpecification'].get('SecurityGroupIds'):
                self.launch_config['LaunchSpecification']['SecurityGroupIds'] += [security_group.group_id]
            else:
                self.launch_config['LaunchSpecification']['SecurityGroupIds'] = [security_group.group_id]

        common.request_spot_instances(self.project_id, self.launch_config)

    def create_default_role_and_profile(self, policies):
        """ Creates default role and instance profile for EC2 deployment.
        :param policies:                Policies to attach to default role
        """
        role_name = self.project_id + '-default-role'
        common.create_role(role_name, DEFAULT_TRUST_POLICY, *policies)
        common.create_instance_profile(role_name, role_name)

    def create_policies(self, custom_policy):
        """Creates policies for EMR related tasks"""
        policies = []

        # declare default policy settings
        default_policy_name = self.project_id + '-default-policy'
        default_policy_document = DEFAULT_POLICY.format(bucket=self.project_id)
        common.create_policy(default_policy_name, default_policy_document)

        policies.append(next(common.get_policies(default_policy_name)))

        if custom_policy:
            print('Creating custom policy')

            custom_policy_name = self.project_id + '-custom-policy'
            common.create_policy(custom_policy_name, custom_policy)
            policies.append(next(common.get_policies(custom_policy_name)))

        return policies

    def connect(self):
        """Set up port forwarding for ipython notebook server"""
        instance = next(common.get_instances(self.project_id))
        instance_ip = instance.public_ip_address or instance.private_ip_address
        key_file = os.path.join(os.path.expanduser('~'), '.ssh', self.project_id)
        ssh.forward(8888, instance_ip, 'ubuntu', key_file)

    def stop(self):
        """Stop all running instances"""
        dryrun = False
        common.cancel_spot_request(self.project_id, dryrun)

        for instance in common.get_instances(self.project_id):
            common.terminate_instance(instance, dryrun)



"""
Schedule tasks using lambda and cloudwatch
"""

import os

PROJECT = os.environ.get('project_name')


def run(event, context):

    from bokchoi import common
    from bokchoi.ec2 import EC2
    from bokchoi.emr import EMR

    settings = common.load_settings(PROJECT)

    platforms = {'EC2': EC2, 'EMR': EMR}
    platform = platforms.get(settings['Platform'])(PROJECT, settings)

    platform.run()
